==> chia_crypto_utils-1.0.18.tar.gz: lib/src/cat/models/spendable_cat.dart <==
// ignore_for_file: lines_longer_than_80_chars

import 'dart:math';

import 'package:chia_crypto_utils/chia_crypto_utils.dart';

class SpendableCat {
  SpendableCat({
    required this.coin,
    required this.innerPuzzle,
    required this.innerSolution,
    this.extraDelta = 0,
  });
  CatCoin coin;
  Program innerPuzzle;
  Program innerSolution;
  int? subtotal;
  int extraDelta;

  Program makeStandardCoinProgram() {
    return Program.list([
      Program.fromBytes(coin.parentCoinInfo),
      Program.fromBytes(innerPuzzle.hash()),
      Program.fromInt(coin.amount),
    ]);
  }

  static void calculateAndAttachSubtotals(List<SpendableCat> spendableCats) {
    final spendInfoMap = <Bytes, SpendableCat>{};
    final deltasMap = <Bytes, int>{};

    // calculate deltas
    for (final spendableCat in spendableCats) {
      final conditionPrograms =
          spendableCat.innerPuzzle.run(spendableCat.innerSolution).program.toList();

      var total = spendableCat.extraDelta * -1;
      for (final createCoinConditionProgram
          in conditionPrograms.where(CreateCoinCondition.isThisCondition)) {
        if (!createCoinConditionProgram.toSource().contains('-113')) {
          final createCoinCondition = CreateCoinCondition.fromProgram(createCoinConditionProgram);
          total += createCoinCondition.amount;
        }
      }
      spendInfoMap[spendableCat.coin.id] = spendableCat;
      deltasMap[spendableCat.coin.id] = spendableCat.coin.amount - total;
    }

    //calculate subtotals
    final subtotalsMap = <Bytes, int>{};
    var subtotal = 0;
    deltasMap.forEach((coinId, delta) {
      subtotalsMap[coinId] = subtotal;
      subtotal += delta;
    });

    final subtotalOffset = subtotalsMap.values.reduce(min);
    final standardizedSubtotals =
        subtotalsMap.map((key, value) => MapEntry(key, value - subtotalOffset));

    // attach subtotals to their respective spendableCat
    // ignore: cascade_invocations
    standardizedSubtotals.forEach((coinId, subtotal) {
      spendInfoMap[coinId]!.subtotal = subtotal;
    });
  }

  @override
  String toString() =>
      'SpendableCat(coin: $coin, innerPuzzle: $innerPuzzle, innerSolution: $innerSolution)';
}

==> chia_crypto_utils-1.0.20.tar.gz: lib/src/cat/models/spendable_cat.dart <==
// ignore_for_file: lines_longer_than_80_chars

import 'dart:math';

import 'package:chia_crypto_utils/chia_crypto_utils.dart';

class SpendableCat {
  SpendableCat({
    required this.coin,
    required this.innerPuzzle,
    required this.innerSolution,
    this.subtotal,
    this.extraDelta = 0,
  });
  final CatCoin coin;
  final Program innerPuzzle;
  final Program innerSolution;
  int? subtotal;
  final int extraDelta;

  Program makeStandardCoinProgram() {
    return Program.list([
      Program.fromAtom(coin.parentCoinInfo),
      Program.fromAtom(innerPuzzle.hash()),
      Program.fromInt(coin.amount),
    ]);
  }

  static void calculateAndAttachSubtotals(List<SpendableCat> spendableCats) {
    final spendInfoMap = <Bytes, SpendableCat>{};
    final deltasMap = <Bytes, int>{};

    // calculate deltas
    for (final spendableCat in spendableCats) {
      final conditionPrograms = spendableCat.innerPuzzle
          .run(spendableCat.innerSolution)
          .program
          .toList();

      var total = spendableCat.extraDelta * -1;
      for (final createCoinConditionProgram
          in conditionPrograms.where(CreateCoinCondition.isThisCondition)) {
        if (!createCoinConditionProgram.toSource().contains('-113')) {
          final createCoinCondition =
              CreateCoinCondition.fromProgram(createCoinConditionProgram);
          total += createCoinCondition.amount;
        }
      }
      spendInfoMap[spendableCat.coin.id] = spendableCat;
      deltasMap[spendableCat.coin.id] = spendableCat.coin.amount - total;
    }

    //calculate subtotals
    final subtotalsMap = <Bytes, int>{};
    var subtotal = 0;
    deltasMap.forEach((coinId, delta) {
      subtotalsMap[coinId] = subtotal;
      subtotal += delta;
    });

    final subtotalOffset = subtotalsMap.values.reduce(min);
    final standardizedSubtotals =
        subtotalsMap.map((key, value) => MapEntry(key, value - subtotalOffset));

    // attach subtotals to their respective spendableCat
    // ignore: cascade_invocations
    standardizedSubtotals.forEach((coinId, subtotal) {
      spendInfoMap[coinId]!.subtotal = subtotal;
    });
  }

  @override
  String toString() =>
      'SpendableCat(coin: $coin, innerPuzzle: $innerPuzzle, innerSolution: $innerSolution)';
}
